 {~ Calendar, time, some astronomical co-ordinate transformations ~}

{$N+}
  { Has to be converted to the appropriate code page;
   the original one is 'Kamenicky'
   -- this is because of Czech names of months and of the Monday .. Sunday}

unit Solar_Ut;
    {uses a part of an old Tomas Rezek's program SUN_BY_NAKED_EYES;}
(*  Copyright (C) 1999 Jan Hollan;
  by "program" this unit is further meant.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

interface

uses {just Jan Hollan's units:}
 Angles_o, {angular and coordinate operations}
 Un_str_m, {instead of Unit_str by Blaise Computing., Inc}
 str_num;

const

 {You may change the whole set of the following FIVE values to fit your site,
  with no need to rename the resulting executable files:}

 {The original values hold for my home, Brno in the Czech Republic:}
 MyLocation_name                 = 'Brno';
 MyLocation_longitude_in_degrees = 16.6 ;  {eastern is positive}
 MyLocation_latitude_in_degrees  = 49.2 ;  {northern is positive}
 MyLocation_longitude_string     ='16.60';
 MyLocation_latitude_string      ='49.20';

 month_cz: array[1..12] of string[8]=
  ('leden','nor','bezen','duben','kvten','erven',
   'ervenec','srpen','z','jen','listopad','prosinec');
 month_en: array[1..12] of string[9]=
  ('January','February','March','April','May','June',
   'July','August','September','October','November','December');
 month_length: array[1..12] of byte = (31,28,31,30,31,30,31,31,30,31,30,31);
 DaysOfWeekCz: array [0..6] of string[7]=
  ('Pondl','ter  ','Steda ','tvrtek','Ptek  ','Sobota ','Nedle ');
 DaysOfWeekEn: array [0..6] of string[9]=
  ('Monday   ','Tuesday  ','Wednesday','Thursday ','Friday   ','Saturday ','Sunday   ');
 _English=1;
 _Czech=2;
 t_UT : real=1;  { (used Time - UTC) / 1 h; 2 for April to October }
 JD_MJD:real=2400000.5; {difference of JD and MJD}
 JD2000=2451545;    {Julian Date of the moment <2000-01-01, 12 h UTC>,
                     abbreviated as J2000.0 }
 JulCentury=36525;  {length of Julian century / 1 day}
 speed_of_light=299792458.0;    {m/s : from definition of 1 m}
 AU=1.49597870E11; {m   : from observation and by fixed Gaussian grav. const.}
 Earth_radius_equ=6378140; {m}
 ForcedJulian:boolean=false;
 ForcedGregorian:boolean=false;
 pressure_db_101325:real=0.97;
 Celsius_temp:real=10;
 Relative_humidity:real=70;
 Elevate_by_refraction:boolean=false;
 Sidereal2SolarTime=1.0027379093;
 refract_multiplier_unset:boolean=true;
VAR
 Lambda,Fi,       {topocentric coordinates in radians}
 sin_Fi, cos_Fi,
 UT,             {Universal time in hours}
 JD,  {Julian date = JD0(YYYY,MM,ND)+UT/24, set this way by Sid_Time_Loc }
 MJD, {Modified Julian Date = JD - 2 400 000.5; enables handling seconds
                  within 11 digits of real type}
 SM,             {Local sidereal time in hours, given by Sid_Time_Loc}

 Epsi,Epsi_rad,  {angle equator-ecliptics; originally for the year 2000.0,
              the former in degrees, the latter in radians}
      CEpsi,SEpsi,  {Cosine and sine of Epsi}
 EL,             {ecliptical longitude}
 RA, Decl,       {Right Ascension and Declination}
 A,h,PA,Air_mass, {Azimuth, ang. height, angle zenith - north pole, ...}
               {These angles are expressed in degrees}
 refract_multiplier    : REAL;

 YYYY:integer;
 MM :word;
 ND:integer; {date}
 DD, HH, MI, SS, HS : word; {day and time}
 Datum,Time: string;
 month_begin: array[1..12] of word;


Function Sun_Long(JDat:real):real; {solar ecliptical longitude}
Function Epsilon(JDat:real):real;
 {tilt of the equator to the ecliptics, gives also its cos and sin
                          ("mean" equator- not considering nutation)}
Function  JD0(YYYY:integer;MM:word;ND:integer):real; {Julian date for 0 UT}
procedure MJD_now;
PROCEDURE Ecl_to_Equ(EL:real; var RA, Decl:real); {just for Ecl.width=0}
Procedure LB_RD(l,b:real; var ra,de:real); {for any Ecl.width}
Procedure Sid_time_loc(var SM_l,JDat:real; UT,Lambda:real);
Function  Get_SM(JDat,Lambda:real):real;   {gives local sidereal time}
procedure Get_JD; {YYYY, MM, DD, HH, MI, SS, HS are to be set for it!}
function YYYYreal(JDat:real):real;
function  Refract_d(h,pr,t,w:real):real;
 {gives refracted (larger) angular height.
  h is non-refracted height, pr is pressure relative to 101325 kPa,
  t is Celsius air temperature, w is relative air humidity}

 {The procedure has just a [pr,t,w] factor (allowing for chanes
 in n-1 probably) by which the function of vacuum angular height
 is multiplied. This is clearly not adequate for low angular
 heigts, at the weather influence (changed n-1) doubles for
 planar atmosphere there, but then again diminishes probably for
 very low h thanks to curvature of the Earth. The [pr,t,w] factor
 and angular depencence are computed according to Jan Vondrak's
 text in Astronomicka prirucka, p. 27, Academia, Praha 1992,
 for heights below 6.5 degrees the angular dependence is my
 polynomial fit to data of unknown origin (I am not able to find
 them once more), probably for dry atmosphere with zero Celsius
 temperature on the ground, as maximum refraction for zero
 observed height is 36.6'.
 The polynomial fit goes down to -1 degree of observed height}

Procedure A_h   (SM_l,RA,Decl:real; var A,h            :real) ;
  {from local Sidereal tiMe, Right Ascension and Declination:
                                        Azimuth and height is computed}
Procedure A_h_A (SM_l,RA,Decl:real; var A,h,   Air_mass_l:real) ;
                                              {Air_mass is
 computed according to simple formula given in Dan Green's article
 in the Int. Comet Quarterly in July 1992, he calls it Rozenberg's
 (1966) equation. It is a function of the refracted h only. There
 should be surely some depencence on refraction angle included,
 but I hope that the dependence is small, as different refraction
 causes change of the path mainly in those layers of atmosphere,
 where the refraction index nears 1 and whose mass may be
 neglected. A small positive contribution for the air mass due to
 larger n-1 should be present, but I do not know its value.}

Procedure A_h_PA(SM_l,RA,Decl:real; var A,h,PA,Air_mass_l:real);
                                           {PA is the position angle of zenith}

Procedure h2air_mass(h:real); {sets Air_Mass from refracted h; 
      gives 0(!) for h<=0}
Function h2t(h,Decl:real; var A,t:real; var why:string):boolean;
  {hour angle and Azimuth for the given Declination and angular height}
Function DoW(JDat,t_UTC:real;language:byte):string;
         {Day of Week, sets local YYYY,MM,ND,HH,Mi,SS,HS,UT and Time string}
Function DoWE(JDat,t_UTC:real):string;
         {Day of Week, in English}
Function DoWN(JDat,t_UTC:real):byte;
         {Day of Week, Monday=1,..., Sunday=7}
Function SM_DoY2T(SM_l,t_UTC,LambdD:real;DoY:word):real;
 {For a given local sidereal time SM and
        ordinal number of a day in a year DoY (1 to 365)
    it gives a time T when SM holds. (LambdD is rad_to_deg(Lambda)).
  It does not take into account, which year is concerned - in decades
  around 2000 the error is less than 3 min.}
Function SM_T2DoY(SM_l,T,t_UTC,LambdD:real):word;
 {like the preceding one, from SM and T it gives the DoY -
  with a maximum error of one day}
Function At_height(MoE:char;h:real;var MJD_aux:real):boolean;
 {gives MJD of Sun passing angular height h in the morning or evening
             (depending on MoE: 'M' or 'E');
  false if Sun does not cross that height at all}
Function TT_UTC(JD_p:real):integer;
 {Approximate difference of Terestric Time (or Ephemeris Time prior to 1984)
  from Universal Time Coordinated (known as Greenwich Mean Time long ago)
  / 1 second}
Function TT_UTC_in_days(JD_p:real):real; {TT_UTC/3600/24}


implementation
var j:byte;

Function Sun_Long(JDat:real):real; {solar ecliptical longitude}
VAR NJD,MS: REAL;
BEGIN
 NJD:=JDat-2.451545E6;
 MS:=deg_to_rad(357.528)+deg_to_rad(0.9856003)*NJD;
 Sun_Long:=Bas_int( (280.466 )+(0.985647352)*NJD
                  + (1.915-0.0047*(NJD/36525))*SIN(MS)
                  + (0.02)*SIN(2*MS)        ,360);
  {more accuracy is obsolete within plus minus two millenia, as greater
   influence
     (up to 8 millidegrees sometimes, with a SD of 2.6 millidegrees)
   have Jupiter, Moon, Mars and Venus together:
   use unit Planets instead for such more accurate computation}
END;

Function Epsilon(JDat:real):real;
      {tilt of the equator to the ecliptics
                       ("mean" equator- not considering nutation)}
var aux:real;
begin
 aux:=(JDat-JD2000)/JulCentury;
 aux:=23.4392911+aux*(-0.0130042+aux*(-1.64e-6+aux*539e-9));
 Epsilon:=aux;
 Epsi_rad:=deg_to_rad(aux);
 CEpsi:=cos(Epsi_rad);
 Sepsi:=sin(Epsi_rad);
end;

Function JD0(YYYY:integer;MM:word;ND:integer):real;
var Y,M,D:real; B: integer;
begin
 Y:=yyyy;
 m:=mm;
 d:=nd;
 if MM in [1,2] then
  begin
   Y:=Y-1;
   M:=M+12;
  end;
 if ((not ForcedJulian)
     and ((YYYY>1582)
        or ((YYYY=1582)
          and ((MM>10)
               or ((MM=10) and (ND>14))
              )
          )
      ))
 or  ForcedGregorian
          then
  begin
   B:=Trunc(Y/100);
   B:= 2 - B + Trunc(B/4.0);
  end
 else B:=0;
 D:=Trunc((M+1)*30.6001)+D+1720994.5+B;
 if Y<0 then
  JD0:=D+Trunc(Y*365.25 - 0.75)
 else
  JD0:=D+Trunc(Y*365.25)
END;

procedure MJD_now;
var Y:word;
begin
 Datum:=__DateStr(Y,MM,DD);
 YYYY:=Y;
 Time:=__TimeStr(HH,MI,SS,HS);
 Get_JD;
end;


PROCEDURE Ecl_to_Equ(EL:real; var RA, Decl:real);
var CEL,SEL : real;
BEGIN
 EL:=deg_to_rad(EL);
 CEL:= cos(EL);
 SEL:= sin(EL);
 if cel=0 then if sel>0 then ra:=90 else ra:=270
 else RA:=rad_to_deg(ARCTAN(CEPSI*SEL/CEL));
 IF CEL<0 THEN RA:=RA+180;
 IF (SEL<0)  AND  (CEL>0)  THEN RA:=RA+360;
 Decl:=SEPSI*SEL;
 Decl:=rad_to_deg(ARCTAN(Decl/(SQRT(1-sqr(Decl)))));
END;

Procedure LB_RD(l,b:real; var ra,de:real);
var Ve:v3; Ma:M3;
begin
 P3s_Vector(deg_to_rad(l),deg_to_rad(b),Ve);
 Mat(Ma,1,-epsi_rad);
 Mat_Vect(Ma,Ve);           {from Ecliptical to Equatorial}
 Vector_P3s(Ve,ra,de);
 ra:=rad_to_deg(ra);
 de:=rad_to_deg(de);
end;

function YYYYreal(JDat:real):real;
begin
 YYYYreal:=100*(Jdat-JD2000)/JulCentury + 2000.0;
end;


Procedure Sid_time_loc(var SM_l,JDat:real; UT,Lambda:real);
var T3:real;
begin
  JDat:=JD0(YYYY,MM,ND);
  T3:=(JDat-JD2000)/JulCentury;
  MJD:=JDat-JD_MJD;
  MJD:=MJD+UT/24;
  JDat:=MJD+JD_MJD;
  SM_l:=Bas_int
    (6.697374558
     + T3*(2400.05133691
           + T3*(0.0000258622
                 - T3*0.0000000017))
     +  Sidereal2SolarTime*UT
     + rad_to_deg(LAMBDA)/15
     , 24);
end;

Function  Get_SM(JDat,Lambda:real):real;
var T3:real;
begin
  T3:=(trunc(JDat+0.5)-JD2000-0.5)/JulCentury;
  Get_SM:=Bas_int
    (6.697374558
     + T3*(2400.05133691
           + T3*(0.0000258622
                 - T3*0.0000000017))
     + frac(JDat+0.5)*24.0657098230
     + rad_to_deg(LAMBDA)/15
     , 24);
end;

Function SM_DoY2T(SM_l,t_UTC,LambdD:real;DoY:word):real;
 {For a given local sidereal time SM and
        ordinal number of a day in a year DoY (1 to 365)
    it gives a time T when SM holds.
  It does not take into account, which year is concerned - in decades
  around 2000 the error is less than 3 min.}
var SM_delt,T_delt:real;
BEGIN
 SM_delt:=     {increment of sidereal time from the start of an UT day}
  SM_l               {local sidereal time / 1 h}
  -LambdD/15       {from local to universal}
  - (6.722         {Sidereal time in years around 2000 at Jan:1 0:00 UTC}
     +(DoY-1)*0.06571 {shift of sidereal time from T that happened
                   from Jan:1 0:00 UTC,}
     );
 T_delt:=
  Bas_Int(SM_delt,24)
  /Sidereal2SolarTime
  +t_UTC;
 if T_delt>24 then T_delt:=T_delt-24.0027379093
 else if T_delt<0 then T_delt:=T_delt+24.0027379093;
 SM_DoY2T:= T_delt;
END;

Function SM_T2DoY(SM_l,T,t_UTC,LambdD:real):word;
 {like the preceding one, from SM and T it gives the DoY -
  with a maximum error of one day}
BEGIN
  SM_T2DoY:=round(Bas_Int
            ((SM_l-LambdD/15-(T-t_UTC)* Sidereal2SolarTime-6.722)
            /0.06571 + 1 + t_UTC/24,365));
END;

procedure Get_JD;
begin
  UT:=(((HS/100.0+ss)/60.0+mi)/60.0+hh-t_UT);
  ND:=DD;
  if UT<0  then dec(ND);
  if UT>=24 then inc(ND);
  UT:=Bas_int(UT,24);
  Sid_time_loc(SM,JD,UT,Lambda);
end;

function  Refract_d(h,pr,t,w:real):real;
 {gives refracted (larger) angular height.
  h is non-refracted height, pr is pressure relative to 101325 kPa,
  t is Celsius air temperature, w is relative air humidity}
var x, {refraction angle / 1 degree as a function of angular height}
    x1 {water vapour pressure / 1 hPa as a function of relative
           humidity and temperature, acc. to Jan Vondrak}
     :real;
begin
 if (h>6.5) then
  begin
   x:=deg_to_rad(h);
   x:=cos(x)/sin(x);
   x:=(60.236-0.070*(x*x))*x/3600  {acc. to Jan Vondrak}
  end
 else
  if h>-2 then
   {a polynomial fit for data of unknown origin, probably for dry
    atmosphere and zero Celsius temperature : }
   x:=(5.0528E+02 + h*(-1.5278E+02 + h*( 2.9775E+01 + h*(-3.3643E+00 + h*(
         1.6082E-01)))))/1000
  else x:=0; {no refraction below -2 degrees of vacuum angular height}
 if refract_multiplier_unset then
  begin
   x1:=1E-4*w*(605+t*(44.94+t*(1.0279+t*(0.0279+t*0.00034))));
    {humidity and temperature dependence of refraction acc. to Jan Vondrak:}
   refract_multiplier:=pr*(1+x1*(-0.000152-x1*0.00000550))/(1+0.00368*t);
  end;
 refract_d:=h+x*refract_multiplier;

end;


PROCEDURE A_h_PA_gen(SM_l,RA,Decl:real; var A,h,PA,Air_mass_l:real;
  PA_too,Air_mass_too:boolean) ;
 VAR
   D, CH, SH, CD, SD, T, ST, CT, h_obs : real ;

BEGIN
   T   := 15 * deg_to_rad(Bas_int(SM_l-(RA/15),24));
   D   := deg_to_rad(Decl);
   ST := sin (T) ; CT := cos (T);
   SD := sin (D) ; CD := cos (D);
   SH := Sin_Fi * SD  +  Cos_Fi * CD * CT;
   {CH := sqrt (1 - SH*SH);
    if abs(CH)<1E-20 then
               : testing earlier from 0103 to prevent neg.arg. of sqrt}
   if abs(SH)>1 - 1E-10 then {near zenith or nadir}
    begin
     PA:=0; A:=0;
     if SH>0 then
      begin h:=90; Air_mass_l:=1; end
     else
      begin h:=-90; Air_mass_l:=1; end
    end
   else
    begin
      CH := sqrt (1 - SH*SH);
      h:= rad_to_deg(ArcTan (SH/CH));
      if Air_mass_too then
       if SH<-0.02 then
        begin Air_Mass_l:=0; h_obs:=h; end
       else
        begin
         h_obs:=refract_d(h,pressure_db_101325,Celsius_temp,Relative_humidity);
         SH:=sin(deg_to_rad(h_obs));
         Air_mass_l:=1/(SH+0.025*exp(-11*SH));  {acc. to Dan Green who cites
                                                it as Rozenberg(1966)}
        end;
     if Elevate_by_refraction then
      if Air_mass_too then
       h:=h_obs
      else
       h:=refract_d(h,pressure_db_101325,Celsius_temp,Relative_humidity);
     if PA_too then
      PA:=C2_to_Pdf(( CD * Sin_Fi - SD * Cos_Fi * CT) / CH, Cos_Fi * ST / CH);
     A :=C2_to_Pdf((-Cos_Fi * SD + Sin_Fi * CD * CT) / CH, CD * ST / CH);
    end;
end;

Procedure A_h(SM_l,RA,Decl:real; var A,h:real) ;
var PA, AM :real;
begin
 A_h_PA_gen(SM_l,RA,Decl,A,h,PA,AM,false,false);
end;

Procedure A_h_A(SM_l,RA,Decl:real; var A,h,Air_mass_l:real) ;
var PA :real;
begin
 A_h_PA_gen(SM_l,RA,Decl,A,h,PA,Air_mass_l,false,true);
end;

Procedure A_h_PA(SM_l,RA,Decl:real; var A,h,PA,Air_mass_l:real) ;
begin
 A_h_PA_gen(SM_l,RA,Decl,A,h,PA,Air_mass_l,true,true);
end;

Procedure h2air_mass(h:real);
 var SH:real;
 begin
   if h>0 then
    begin
     SH:=sin(deg_to_rad(h));
     Air_mass:=1/(SH+0.025*exp(-11*SH));
    end
   else
    air_mass:=0;
 end;

Function h2t(h,Decl:real; var A,t:real; var why:string):boolean;
  {hour angle and Azimuth for the given Declination and angular height}
var  D, CH, SH, SA, CD, SD, CA, ST, CT, T_aux, sigFi : real ;
begin
   D  := deg_to_rad(Decl);
   h2t:=true; why:='';
   h:=deg_to_rad(h);
   if Fi>0 then sigFi:=1 else sigFi:=-1;
   if abs(Fi)-pi/2+D*sigFi>h then
    begin
     h2t:=false;
     why:='always higher';
     exit;
    end;
   if -abs(Fi)+pi/2+D*sigFi<h then
    begin
     h2t:=false;
     why:='always lower';
     exit;
    end;
   SD := sin (D) ;  CD := cos (D);
   SH := sin(h);
   CH := cos(h);
   CT := (SH - Sin_Fi * SD) / (Cos_Fi * CD);
   ST := sqrt(1-Sqr(CT));
   if abs(CT)<0.01 then
    T_aux := 90.0 * (1 - 2*ct/pi)
   else
    T_aux := rad_to_deg(ArcTan(ST/CT));
   if T_aux<0 then T_aux:=T_aux+180;
   T:= T_aux/15;
   SA := CD * ST / CH;
   CA := (-Cos_Fi * SD  +  Sin_Fi * CD * CT ) / CH ;
   A  := rad_to_deg(ARCTAN(SA/CA));
   IF CA <0 THEN A:=A+180;
   IF (SA <0)  AND  (CA >=0)  THEN A:=A+360;
end;

Function DoW(JDat,t_UTC:real;language:byte):string;
var Z,B,C,E : longint;
begin
Z:=DoWN(JDat,t_UTC)-1;
 {(7+trunc(JDat+t_UTC/24-2377892.49999) mod 7) mod 7;}
                           { not .50000  to behave better at midnight
                              - just the 0.84 of last second becomes tomorrow   }
if language=2 then
 DoW:=DaysOfWeekCz[Z]
else
 DoW:=DaysOfWeekEn[Z];

Z:=trunc(JDat+t_UTC/24+0.50001);
if ((Z<2299163) and (not ForcedGregorian)) or ForcedJulian then
 B:=Z+1524
else
 begin
  B:=trunc((Z-1867216.25)/36524.25);
  B:=Z+1+B-trunc(B/4.0)+1524;
 end;
C:=trunc((B-122.1)/365.25);
Z:=trunc(365.25*C);
E:=trunc((B-Z)/30.6001);
ND:=B-Z-trunc(30.6001*E);
MM:=E-1;
if E>13 then MM:=MM-12;
YYYY:=C-4716;
if MM<3 then inc(YYYY);
UT:=24*frac(JDat+0.5);
Time:=h_min_s(24*frac(JDat+t_UTC/24+0.5));
if __2TimeStr(Time,HH,Mi,SS,HS) then;
end;

Function DoWE(JDat,t_UTC:real):string;
         {Day of Week, in English}
begin
DoWE:=DaysOfWeekEn[{(7+trunc(JDat+t_UTC/24-2377892.5) mod 7) mod 7}
                    DoWN(JDat,t_UTC)-1];
end;

Function DoWN(JDat,t_UTC:real):byte;
         {Day of Week, in English}
begin
DoWN:=1 + trunc(JDat + t_UTC/24 - {2377892.49999}
                                  6.49999) mod 7;
                             { not .50000  to behave better at midnight
                              - just the 0.84 of last second becomes tomorrow   }
end;

Function At_height(MoE:char;h:real;var MJD_aux:real):boolean;
 {gives MJD of Sun passing angular height h in the morning or evening
             (depending on MoE: 'M' or 'E');
  false if Sun does not cross that height at all}
var t_Sun,t_twi{,Fi_deg}:real; aux_boo:boolean; aux_s:string;
begin
 if MJD_aux<0 then MJD_aux:=MJD_aux-1.0;
 MJD_aux:=int(MJD_aux)+0.25-Lambda/(2*pi)
                            {t_UT/24 originally, what is unsafe};
                            {first approx. of the morning}
 if MoE='E' then MJD_aux:=MJD_aux+0.5;
 Ecl_to_Equ(Sun_Long(MJD_aux+JD_MJD), RA, Decl);
 aux_boo:=h2t(h,Decl,A,T_twi,aux_s);
 if not aux_boo then
  {Fi_deg:=rad_to_deg(fi);
  if ((90-Decl)<(Fi_deg-h))
  or ((90-Decl)>(180-Fi_deg-h)) then
  }
  At_height:=false
 else
  begin
   t_sun:=Bas_Int((Get_SM(MJD_aux+JD_MJD,lambda) - RA/15),24);
   {T_twi:=T_h(h,Decl,A,);}
   if MoE='M' then T_twi:=24-T_twi;
   MJD_aux:= MJD_aux+(t_twi-t_sun)/24;
   At_height:=true;
  end;
end;

Function TT_UTC(JD_p:real):integer;
 {Approximate difference of Terestric Time (or Ephemeris Time prior to 1984)
  from Universal Time Coordinated (known as Greenwich Mean Time long ago)
  / 1 second}
 var y,yo:{real} {double} extended;
begin
 y:=JD0(1984,01,01); yo:=y;
 if (JD_p>y) and (JD_p<y+18000) then
  begin
   y:=JD0(1970,01,01);
   TT_UTC:=roundUp(41.9+0.77*(JD_p-y)/365.25);
   exit;
  end;
 y:=JD0(1900,01,01);
 if (JD_p>y) and (JD_p<=yo) then
  begin
   y:=(JD_p-y)/365.25;
   TT_UTC:=roundUp(
    -7.18E+0000 + y*(
     2.44E+0000 + y*(
    -7.45E-0002 + y*(
     1.02E-0003 + y*
    -4.53E-0006 )))
    );
  end
 else
  begin                {a crude polynomial approximation of data by Jan Manek}
   y:=JD0(0,01,01);
   y:=(JD_p-y)/365.25;
   TT_UTC:=roundUp(
     1.394E4   + y*(
    -20.50     + y*(
     0.009323  + y*(
    -7.594E-7  + y*(
    -4.107E-10 + y*
     7.723E-14      ))))
                 );
  end
end;

Function TT_UTC_in_days(JD_p:real):real; {TT_UTC/3600/24}
begin
 TT_UTC_in_days:=TT_UTC(JD_p)/3600/24;
end;


begin

Lambda:=deg_to_rad(MyLocation_longitude_in_degrees);
Fi:=deg_to_rad(MyLocation_latitude_in_degrees);

sin_Fi:=sin(fi); cos_Fi:=cos(fi);
     {initialisation of Epsi for J2000.0, see definition of JD2000 in const}
JD:=JD2000;
Epsi:=Epsilon(JD);
     {and of remaining times for the system time};
MJD_now;
if MM in [4..10] then t_UT:=2;
MJD_now;

month_begin[1]:=1;  {and for a non-leap year:}
for j:= 2 to 12 do
 month_begin[j]:=month_begin[j-1]+month_length[j-1];

END.
